@@ -1,17 +1,20 @@
 //! renaming of branches
 
-use crate::{error::Result, sync::utils};
+use crate::{
+	error::Result,
+	sync::{repository::repo, RepoPath},
+};
 use scopetime::scope_time;
 
 /// Rename the branch reference
 pub fn rename_branch(
-	repo_path: &str,
+	repo_path: &RepoPath,
 	branch_ref: &str,
 	new_name: &str,
 ) -> Result<()> {
-	scope_time!("delete_branch");
+	scope_time!("rename_branch");
 
-	let repo = utils::repo(repo_path)?;
+	let repo = repo(repo_path)?;
 	let branch_as_ref = repo.find_reference(branch_ref)?;
 	let mut branch = git2::Branch::wrap(branch_as_ref);
 	branch.rename(new_name, true)?;
@@ -29,7 +32,8 @@ mod test {
 	fn test_rename_branch() {
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		create_branch(repo_path, "branch1").unwrap();
 
