@@ -1,21 +1,102 @@
-use super::{repository::repo, utils::work_dir, RepoPath};
-use crate::error::{Error, Result};
+use super::{repository::repo, RepoPath};
+use crate::error::{self, Result};
 use scopetime::scope_time;
 use std::{
 	fs::File,
 	io::{Read, Write},
-	path::Path,
+	path::{Path, PathBuf},
 	process::Command,
+	str::FromStr,
 };
 
-const HOOK_POST_COMMIT: &str = ".git/hooks/post-commit";
-const HOOK_PRE_COMMIT: &str = ".git/hooks/pre-commit";
-const HOOK_COMMIT_MSG: &str = ".git/hooks/commit-msg";
-const HOOK_COMMIT_MSG_TEMP_FILE: &str = ".git/COMMIT_EDITMSG";
+const HOOK_POST_COMMIT: &str = "post-commit";
+const HOOK_PRE_COMMIT: &str = "pre-commit";
+const HOOK_COMMIT_MSG: &str = "commit-msg";
+const HOOK_COMMIT_MSG_TEMP_FILE: &str = "COMMIT_EDITMSG";
+
+struct HookPaths {
+	git: PathBuf,
+	hook: PathBuf,
+	pwd: PathBuf,
+}
+
+impl HookPaths {
+	pub fn new(repo_path: &RepoPath, hook: &str) -> Result<Self> {
+		let repo = repo(repo_path)?;
+		let pwd = repo
+			.workdir()
+			.unwrap_or_else(|| repo.path())
+			.to_path_buf();
+
+		let git_dir = repo.path().to_path_buf();
+		let hooks_path = repo
+			.config()
+			.and_then(|config| config.get_string("core.hooksPath"))
+			.map_or_else(
+				|e| {
+					log::error!("hookspath error: {}", e);
+					repo.path().to_path_buf().join("hooks/")
+				},
+				PathBuf::from,
+			);
+
+		let hook = hooks_path.join(hook);
+
+		let hook = shellexpand::full(
+			hook.as_os_str()
+				.to_str()
+				.ok_or(error::Error::PathString)?,
+		)?;
+
+		let hook = PathBuf::from_str(hook.as_ref())
+			.map_err(|_| error::Error::PathString)?;
+
+		Ok(Self {
+			git: git_dir,
+			hook,
+			pwd,
+		})
+	}
+
+	pub fn is_executable(&self) -> bool {
+		self.hook.exists() && is_executable(&self.hook)
+	}
+
+	/// this function calls hook scripts based on conventions documented here
+	/// see <https://git-scm.com/docs/githooks>
+	pub fn run_hook(&self, args: &[&str]) -> Result<HookResult> {
+		let arg_str = format!("{:?} {}", self.hook, args.join(" "));
+		let bash_args = vec!["-c".to_string(), arg_str];
+
+		log::trace!("run hook '{:?}' in '{:?}'", self.hook, self.pwd);
+
+		let output = Command::new("bash")
+			.args(bash_args)
+			.current_dir(&self.pwd)
+			// This call forces Command to handle the Path environment correctly on windows,
+			// the specific env set here does not matter
+			// see https://github.com/rust-lang/rust/issues/37519
+			.env(
+				"DUMMY_ENV_TO_FIX_WINDOWS_CMD_RUNS",
+				"FixPathHandlingOnWindows",
+			)
+			.output()?;
+
+		if output.status.success() {
+			Ok(HookResult::Ok)
+		} else {
+			let err = String::from_utf8_lossy(&output.stderr);
+			let out = String::from_utf8_lossy(&output.stdout);
+			let formatted = format!("{}{}", out, err);
+
+			Ok(HookResult::NotOk(formatted))
+		}
+	}
+}
 
 /// this hook is documented here <https://git-scm.com/docs/githooks#_commit_msg>
 /// we use the same convention as other git clients to create a temp file containing
-/// the commit message at `.git/COMMIT_EDITMSG` and pass it's relative path as the only
+/// the commit message at `<.git|hooksPath>/COMMIT_EDITMSG` and pass it's relative path as the only
 /// parameter to the hook script.
 pub fn hooks_commit_msg(
 	repo_path: &RepoPath,
@@ -23,18 +104,17 @@ pub fn hooks_commit_msg(
 ) -> Result<HookResult> {
 	scope_time!("hooks_commit_msg");
 
-	let work_dir = work_dir_as_string(repo_path)?;
+	let hooks_path = HookPaths::new(repo_path, HOOK_COMMIT_MSG)?;
 
-	if hook_runable(work_dir.as_str(), HOOK_COMMIT_MSG) {
-		let temp_file = Path::new(work_dir.as_str())
-			.join(HOOK_COMMIT_MSG_TEMP_FILE);
+	if hooks_path.is_executable() {
+		let temp_file =
+			hooks_path.git.join(HOOK_COMMIT_MSG_TEMP_FILE);
 		File::create(&temp_file)?.write_all(msg.as_bytes())?;
 
-		let res = run_hook(
-			work_dir.as_str(),
-			HOOK_COMMIT_MSG,
-			&[HOOK_COMMIT_MSG_TEMP_FILE],
-		)?;
+		let res = hooks_path.run_hook(&[temp_file
+			.as_os_str()
+			.to_string_lossy()
+			.as_ref()])?;
 
 		// load possibly altered msg
 		msg.clear();
@@ -51,10 +131,10 @@ pub fn hooks_commit_msg(
 pub fn hooks_pre_commit(repo_path: &RepoPath) -> Result<HookResult> {
 	scope_time!("hooks_pre_commit");
 
-	let work_dir = work_dir_as_string(repo_path)?;
+	let hook = HookPaths::new(repo_path, HOOK_PRE_COMMIT)?;
 
-	if hook_runable(work_dir.as_str(), HOOK_PRE_COMMIT) {
-		Ok(run_hook(work_dir.as_str(), HOOK_PRE_COMMIT, &[])?)
+	if hook.is_executable() {
+		Ok(hook.run_hook(&[])?)
 	} else {
 		Ok(HookResult::Ok)
 	}
@@ -63,35 +143,15 @@ pub fn hooks_pre_commit(repo_path: &RepoPath) -> Result<HookResult> {
 pub fn hooks_post_commit(repo_path: &RepoPath) -> Result<HookResult> {
 	scope_time!("hooks_post_commit");
 
-	let work_dir = work_dir_as_string(repo_path)?;
-	let work_dir_str = work_dir.as_str();
+	let hook = HookPaths::new(repo_path, HOOK_POST_COMMIT)?;
 
-	if hook_runable(work_dir_str, HOOK_POST_COMMIT) {
-		Ok(run_hook(work_dir_str, HOOK_POST_COMMIT, &[])?)
+	if hook.is_executable() {
+		Ok(hook.run_hook(&[])?)
 	} else {
 		Ok(HookResult::Ok)
 	}
 }
 
-fn work_dir_as_string(repo_path: &RepoPath) -> Result<String> {
-	let repo = repo(repo_path)?;
-	work_dir(&repo)?
-		.to_str()
-		.map(std::string::ToString::to_string)
-		.ok_or_else(|| {
-			Error::Generic(
-				"workdir contains invalid utf8".to_string(),
-			)
-		})
-}
-
-fn hook_runable(path: &str, hook: &str) -> bool {
-	let path = Path::new(path);
-	let path = path.join(hook);
-
-	path.exists() && is_executable(&path)
-}
-
 ///
 #[derive(Debug, PartialEq)]
 pub enum HookResult {
@@ -101,48 +161,19 @@ pub enum HookResult {
 	NotOk(String),
 }
 
-/// this function calls hook scripts based on conventions documented here
-/// see <https://git-scm.com/docs/githooks>
-fn run_hook(
-	path: &str,
-	hook_script: &str,
-	args: &[&str],
-) -> Result<HookResult> {
-	let arg_str = format!("{} {}", hook_script, args.join(" "));
-	let bash_args = vec!["-c".to_string(), arg_str];
-
-	let output = Command::new("bash")
-		.args(bash_args)
-		.current_dir(path)
-		// This call forces Command to handle the Path environment correctly on windows,
-		// the specific env set here does not matter
-		// see https://github.com/rust-lang/rust/issues/37519
-		.env(
-			"DUMMY_ENV_TO_FIX_WINDOWS_CMD_RUNS",
-			"FixPathHandlingOnWindows",
-		)
-		.output()?;
-
-	if output.status.success() {
-		Ok(HookResult::Ok)
-	} else {
-		let err = String::from_utf8_lossy(&output.stderr);
-		let out = String::from_utf8_lossy(&output.stdout);
-		let formatted = format!("{}{}", out, err);
-
-		Ok(HookResult::NotOk(formatted))
-	}
-}
-
 #[cfg(not(windows))]
 fn is_executable(path: &Path) -> bool {
 	use std::os::unix::fs::PermissionsExt;
 	let metadata = match path.metadata() {
 		Ok(metadata) => metadata,
-		Err(_) => return false,
+		Err(e) => {
+			log::error!("metadata error: {}", e);
+			return false;
+		}
 	};
 
 	let permissions = metadata.permissions();
+
 	permissions.mode() & 0o111 != 0
 }
 
@@ -156,8 +187,9 @@ const fn is_executable(_: &Path) -> bool {
 #[cfg(test)]
 mod tests {
 	use super::*;
-	use crate::sync::tests::repo_init;
+	use crate::sync::tests::{repo_init, repo_init_bare};
 	use std::fs::{self, File};
+	use tempfile::TempDir;
 
 	#[test]
 	fn test_smoke() {
@@ -176,17 +208,29 @@ mod tests {
 		assert_eq!(res, HookResult::Ok);
 	}
 
-	fn create_hook(path: &Path, hook_path: &str, hook_script: &[u8]) {
-		File::create(&path.join(hook_path))
-			.unwrap()
-			.write_all(hook_script)
-			.unwrap();
+	fn create_hook(
+		path: &RepoPath,
+		hook: &str,
+		hook_script: &[u8],
+	) -> PathBuf {
+		let hook = HookPaths::new(path, hook).unwrap();
+
+		let path = hook.hook.clone();
+
+		create_hook_in_path(&hook.hook, hook_script);
+
+		path
+	}
+
+	fn create_hook_in_path(path: &Path, hook_script: &[u8]) {
+		File::create(path).unwrap().write_all(hook_script).unwrap();
 
 		#[cfg(not(windows))]
 		{
 			Command::new("chmod")
-				.args(&["+x", hook_path])
-				.current_dir(path)
+				.arg("+x")
+				.arg(path)
+				// .current_dir(path)
 				.output()
 				.unwrap();
 		}
@@ -203,7 +247,7 @@ mod tests {
 exit 0
         ";
 
-		create_hook(root, HOOK_COMMIT_MSG, hook);
+		create_hook(repo_path, HOOK_COMMIT_MSG, hook);
 
 		let mut msg = String::from("test");
 		let res = hooks_commit_msg(repo_path, &mut msg).unwrap();
@@ -224,7 +268,7 @@ exit 0
 exit 0
         ";
 
-		create_hook(root, HOOK_PRE_COMMIT, hook);
+		create_hook(repo_path, HOOK_PRE_COMMIT, hook);
 		let res = hooks_pre_commit(repo_path).unwrap();
 		assert_eq!(res, HookResult::Ok);
 	}
@@ -241,7 +285,55 @@ echo 'rejected'
 exit 1
         ";
 
-		create_hook(root, HOOK_PRE_COMMIT, hook);
+		create_hook(repo_path, HOOK_PRE_COMMIT, hook);
+		let res = hooks_pre_commit(repo_path).unwrap();
+		assert!(res != HookResult::Ok);
+	}
+
+	#[test]
+	fn test_pre_commit_fail_hookspath() {
+		let (_td, repo) = repo_init().unwrap();
+		let root = repo.path().parent().unwrap();
+		let hooks = TempDir::new().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
+
+		let hook = b"#!/bin/sh
+echo 'rejected'        
+exit 1
+        ";
+
+		create_hook_in_path(&hooks.path().join("pre-commit"), hook);
+		repo.config()
+			.unwrap()
+			.set_str(
+				"core.hooksPath",
+				hooks.path().as_os_str().to_str().unwrap(),
+			)
+			.unwrap();
+		let res = hooks_pre_commit(repo_path).unwrap();
+		assert_eq!(
+			res,
+			HookResult::NotOk(String::from("rejected\n"))
+		);
+	}
+
+	#[test]
+	fn test_pre_commit_fail_bare() {
+		let (git_root, _repo) = repo_init_bare().unwrap();
+		let workdir = TempDir::new().unwrap();
+		let git_root = git_root.into_path();
+		let repo_path = &RepoPath::Workdir {
+			gitdir: dbg!(git_root.to_path_buf()),
+			workdir: dbg!(workdir.into_path()),
+		};
+
+		let hook = b"#!/bin/sh
+echo 'rejected'        
+exit 1
+        ";
+
+		create_hook(repo_path, HOOK_PRE_COMMIT, hook);
 		let res = hooks_pre_commit(repo_path).unwrap();
 		assert!(res != HookResult::Ok);
 	}
@@ -265,7 +357,7 @@ import sys
 sys.exit(0)
         ";
 
-		create_hook(root, HOOK_PRE_COMMIT, hook);
+		create_hook(repo_path, HOOK_PRE_COMMIT, hook);
 		let res = hooks_pre_commit(repo_path).unwrap();
 		assert_eq!(res, HookResult::Ok);
 	}
@@ -289,7 +381,7 @@ import sys
 sys.exit(1)
         ";
 
-		create_hook(root, HOOK_PRE_COMMIT, hook);
+		create_hook(repo_path, HOOK_PRE_COMMIT, hook);
 		let res = hooks_pre_commit(repo_path).unwrap();
 		assert!(res != HookResult::Ok);
 	}
@@ -307,7 +399,7 @@ echo 'rejected'
 exit 1
         ";
 
-		create_hook(root, HOOK_COMMIT_MSG, hook);
+		create_hook(repo_path, HOOK_COMMIT_MSG, hook);
 
 		let mut msg = String::from("test");
 		let res = hooks_commit_msg(repo_path, &mut msg).unwrap();
@@ -324,7 +416,8 @@ exit 1
 	fn test_hooks_commit_msg_reject_in_subfolder() {
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
-		// let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		let hook = b"#!/bin/sh
 echo 'msg' > $1
@@ -332,7 +425,7 @@ echo 'rejected'
 exit 1
         ";
 
-		create_hook(root, HOOK_COMMIT_MSG, hook);
+		create_hook(repo_path, HOOK_COMMIT_MSG, hook);
 
 		let subfolder = root.join("foo/");
 		fs::create_dir_all(&subfolder).unwrap();
@@ -364,7 +457,7 @@ echo 'msg' > $1
 exit 0
         ";
 
-		create_hook(root, HOOK_COMMIT_MSG, hook);
+		create_hook(repo_path, HOOK_COMMIT_MSG, hook);
 
 		let mut msg = String::from("test");
 		let res = hooks_commit_msg(repo_path, &mut msg).unwrap();
@@ -377,13 +470,15 @@ exit 0
 	fn test_post_commit_hook_reject_in_subfolder() {
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		let hook = b"#!/bin/sh
 echo 'rejected'
 exit 1
         ";
 
-		create_hook(root, HOOK_POST_COMMIT, hook);
+		create_hook(repo_path, HOOK_POST_COMMIT, hook);
 
 		let subfolder = root.join("foo/");
 		fs::create_dir_all(&subfolder).unwrap();
@@ -397,4 +492,77 @@ exit 1
 			HookResult::NotOk(String::from("rejected\n"))
 		);
 	}
+
+	// make sure we run the hooks with the correct pwd.
+	// for non-bare repos this is the dir of the worktree
+	// unfortunately does not work on windows
+	#[test]
+	#[cfg(unix)]
+	fn test_pre_commit_workdir() {
+		let (_td, repo) = repo_init().unwrap();
+		let root = repo.path().parent().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
+		let workdir =
+			crate::sync::utils::repo_work_dir(repo_path).unwrap();
+
+		let hook = b"#!/bin/sh
+echo $(pwd)
+exit 1
+        ";
+
+		create_hook(repo_path, HOOK_PRE_COMMIT, hook);
+		let res = hooks_pre_commit(repo_path).unwrap();
+		if let HookResult::NotOk(res) = res {
+			assert_eq!(
+				Path::new(res.trim_end()),
+				Path::new(&workdir)
+			);
+		} else {
+			assert!(false);
+		}
+	}
+
+	#[test]
+	fn test_hook_pwd_in_bare_without_workdir() {
+		let (_td, _repo) = repo_init_bare().unwrap();
+		let git_root = _repo.path().to_path_buf();
+		let repo_path = &RepoPath::Path(git_root.clone());
+
+		let hook =
+			HookPaths::new(repo_path, HOOK_POST_COMMIT).unwrap();
+
+		assert_eq!(hook.pwd, dbg!(git_root));
+	}
+
+	#[test]
+	fn test_hook_pwd_in_bare_with_workdir() {
+		let (git_root, _repo) = repo_init_bare().unwrap();
+		let workdir = TempDir::new().unwrap();
+		let git_root = git_root.into_path();
+		let repo_path = &RepoPath::Workdir {
+			gitdir: dbg!(git_root.to_path_buf()),
+			workdir: dbg!(workdir.path().to_path_buf()),
+		};
+
+		let hook =
+			HookPaths::new(repo_path, HOOK_POST_COMMIT).unwrap();
+
+		assert_eq!(
+			hook.pwd.canonicalize().unwrap(),
+			dbg!(workdir.path().canonicalize().unwrap())
+		);
+	}
+
+	#[test]
+	fn test_hook_pwd() {
+		let (_td, _repo) = repo_init().unwrap();
+		let git_root = _repo.path().to_path_buf();
+		let repo_path = &RepoPath::Path(git_root.clone());
+
+		let hook =
+			HookPaths::new(repo_path, HOOK_POST_COMMIT).unwrap();
+
+		assert_eq!(hook.pwd, git_root.parent().unwrap());
+	}
 }
