@@ -1,7 +1,7 @@
 use super::{
-	filetree::FileTreeComponent,
+	status_tree::StatusTreeComponent,
 	utils::filetree::{FileTreeItem, FileTreeItemKind},
-	CommandBlocking, DrawableComponent,
+	CommandBlocking, DrawableComponent, SharedOptions,
 };
 use crate::{
 	components::{CommandInfo, Component, EventState},
@@ -11,31 +11,40 @@ use crate::{
 	ui::style::SharedTheme,
 };
 use anyhow::Result;
-use asyncgit::{sync, StatusItem, StatusItemType, CWD};
+use asyncgit::{
+	sync::{self, RepoPathRef},
+	StatusItem, StatusItemType,
+};
 use crossterm::event::Event;
 use std::path::Path;
 use tui::{backend::Backend, layout::Rect, Frame};
 
 ///
 pub struct ChangesComponent {
-	files: FileTreeComponent,
+	repo: RepoPathRef,
+	files: StatusTreeComponent,
 	is_working_dir: bool,
 	queue: Queue,
 	key_config: SharedKeyConfig,
+	options: SharedOptions,
 }
 
 impl ChangesComponent {
 	///
+	//TODO: fix
+	#[allow(clippy::too_many_arguments)]
 	pub fn new(
+		repo: RepoPathRef,
 		title: &str,
 		focus: bool,
 		is_working_dir: bool,
 		queue: Queue,
 		theme: SharedTheme,
 		key_config: SharedKeyConfig,
+		options: SharedOptions,
 	) -> Self {
 		Self {
-			files: FileTreeComponent::new(
+			files: StatusTreeComponent::new(
 				title,
 				focus,
 				Some(queue.clone()),
@@ -45,6 +54,8 @@ impl ChangesComponent {
 			is_working_dir,
 			queue,
 			key_config,
+			options,
+			repo,
 		}
 	}
 
@@ -82,9 +93,15 @@ impl ChangesComponent {
 					let path = Path::new(i.path.as_str());
 					match i.status {
 						StatusItemType::Deleted => {
-							sync::stage_addremoved(CWD, path)?;
+							sync::stage_addremoved(
+								&self.repo.borrow(),
+								path,
+							)?;
 						}
-						_ => sync::stage_add_file(CWD, path)?,
+						_ => sync::stage_add_file(
+							&self.repo.borrow(),
+							path,
+						)?,
 					};
 
 					if self.is_empty() {
@@ -95,17 +112,21 @@ impl ChangesComponent {
 					return Ok(true);
 				}
 
+				let config =
+					self.options.borrow().status_show_untracked;
+
 				//TODO: check if we can handle the one file case with it aswell
 				sync::stage_add_all(
-					CWD,
+					&self.repo.borrow(),
 					tree_item.info.full_path.as_str(),
+					config,
 				)?;
 
 				return Ok(true);
 			}
 
 			let path = tree_item.info.full_path.as_str();
-			sync::reset_stage(CWD, path)?;
+			sync::reset_stage(&self.repo.borrow(), path)?;
 			return Ok(true);
 		}
 
@@ -113,7 +134,9 @@ impl ChangesComponent {
 	}
 
 	fn index_add_all(&mut self) -> Result<()> {
-		sync::stage_add_all(CWD, "*")?;
+		let config = self.options.borrow().status_show_untracked;
+
+		sync::stage_add_all(&self.repo.borrow(), "*", config)?;
 
 		self.queue.push(InternalEvent::Update(NeedsUpdate::ALL));
 
@@ -121,7 +144,7 @@ impl ChangesComponent {
 	}
 
 	fn stage_remove_all(&mut self) -> Result<()> {
-		sync::reset_stage(CWD, "*")?;
+		sync::reset_stage(&self.repo.borrow(), "*")?;
 
 		self.queue.push(InternalEvent::Update(NeedsUpdate::ALL));
 
@@ -146,9 +169,10 @@ impl ChangesComponent {
 
 	fn add_to_ignore(&mut self) -> bool {
 		if let Some(tree_item) = self.selection() {
-			if let Err(e) =
-				sync::add_to_ignore(CWD, &tree_item.info.full_path)
-			{
+			if let Err(e) = sync::add_to_ignore(
+				&self.repo.borrow(),
+				&tree_item.info.full_path,
+			) {
 				self.queue.push(InternalEvent::ShowErrorMsg(
 					format!(
 						"ignore error:\n{}\nfile:\n{:?}",
@@ -192,43 +216,35 @@ impl Component for ChangesComponent {
 		if self.is_working_dir {
 			out.push(CommandInfo::new(
 				strings::commands::stage_all(&self.key_config),
-				some_selection,
-				self.focused(),
+				true,
+				some_selection && self.focused(),
 			));
 			out.push(CommandInfo::new(
 				strings::commands::stage_item(&self.key_config),
-				some_selection,
-				self.focused(),
+				true,
+				some_selection && self.focused(),
 			));
 			out.push(CommandInfo::new(
 				strings::commands::reset_item(&self.key_config),
-				some_selection,
-				self.focused(),
+				true,
+				some_selection && self.focused(),
 			));
 			out.push(CommandInfo::new(
 				strings::commands::ignore_item(&self.key_config),
-				some_selection,
-				self.focused(),
+				true,
+				some_selection && self.focused(),
 			));
 		} else {
 			out.push(CommandInfo::new(
 				strings::commands::unstage_item(&self.key_config),
-				some_selection,
-				self.focused(),
+				true,
+				some_selection && self.focused(),
 			));
 			out.push(CommandInfo::new(
 				strings::commands::unstage_all(&self.key_config),
-				some_selection,
-				self.focused(),
+				true,
+				some_selection && self.focused(),
 			));
-			out.push(
-				CommandInfo::new(
-					strings::commands::commit_open(&self.key_config),
-					!self.is_empty(),
-					self.focused() || force_all,
-				)
-				.order(-1),
-			);
 		}
 
 		CommandBlocking::PassingOn
@@ -241,13 +257,8 @@ impl Component for ChangesComponent {
 
 		if self.focused() {
 			if let Event::Key(e) = ev {
-				return if e == self.key_config.open_commit
-					&& !self.is_working_dir
-					&& !self.is_empty()
+				return if e == self.key_config.keys.stage_unstage_item
 				{
-					self.queue.push(InternalEvent::OpenCommit);
-					Ok(EventState::Consumed)
-				} else if e == self.key_config.enter {
 					try_or_popup!(
 						self,
 						"staging error:",
@@ -258,7 +269,7 @@ impl Component for ChangesComponent {
 						NeedsUpdate::ALL,
 					));
 					Ok(EventState::Consumed)
-				} else if e == self.key_config.status_stage_all
+				} else if e == self.key_config.keys.status_stage_all
 					&& !self.is_empty()
 				{
 					if self.is_working_dir {
@@ -273,11 +284,11 @@ impl Component for ChangesComponent {
 					self.queue
 						.push(InternalEvent::StatusLastFileMoved);
 					Ok(EventState::Consumed)
-				} else if e == self.key_config.status_reset_item
+				} else if e == self.key_config.keys.status_reset_item
 					&& self.is_working_dir
 				{
 					Ok(self.dispatch_reset_workdir().into())
-				} else if e == self.key_config.status_ignore_file
+				} else if e == self.key_config.keys.status_ignore_file
 					&& self.is_working_dir
 					&& !self.is_empty()
 				{
