@@ -1,6 +1,7 @@
 use crate::{components::AppOption, tabs::StashingOptions};
-use asyncgit::sync::{
-	diff::DiffLinePosition, CommitId, CommitTags, TreeFile,
+use asyncgit::{
+	sync::{diff::DiffLinePosition, CommitId, CommitTags, TreeFile},
+	PushType,
 };
 use bitflags::bitflags;
 use std::{
@@ -36,12 +37,15 @@ pub enum Action {
 	ResetLines(String, Vec<DiffLinePosition>),
 	StashDrop(Vec<CommitId>),
 	StashPop(CommitId),
-	DeleteBranch(String, bool),
+	DeleteLocalBranch(String),
+	DeleteRemoteBranch(String),
 	DeleteTag(String),
+	DeleteRemoteTag(String, String),
 	ForcePush(String, bool),
 	PullMerge { incoming: usize, rebase: bool },
 	AbortMerge,
 	AbortRebase,
+	AbortRevert,
 }
 
 ///
@@ -53,6 +57,8 @@ pub enum InternalEvent {
 	///
 	ShowErrorMsg(String),
 	///
+	ShowInfoMsg(String),
+	///
 	Update(NeedsUpdate),
 	///
 	StatusLastFileMoved,
@@ -61,7 +67,7 @@ pub enum InternalEvent {
 	///
 	PopupStashing(StashingOptions),
 	///
-	TabSwitch,
+	TabSwitchStatus,
 	///
 	InspectCommit(CommitId, Option<CommitTags>),
 	///
@@ -83,7 +89,7 @@ pub enum InternalEvent {
 	///
 	OpenExternalEditor(Option<String>),
 	///
-	Push(String, bool, bool),
+	Push(String, PushType, bool, bool),
 	///
 	Pull(String),
 	///
@@ -96,6 +102,8 @@ pub enum InternalEvent {
 	OpenFileFinder(Vec<TreeFile>),
 	///
 	FileFinderChanged(Option<PathBuf>),
+	///
+	FetchRemotes,
 }
 
 /// single threaded simple queue for components to communicate with each other
