@@ -15,8 +15,7 @@ use anyhow::Result;
 use asyncgit::{
 	cached,
 	sync::{
-		self, get_branches_info, status::StatusType, RepoPath,
-		RepoPathRef, RepoState,
+		self, status::StatusType, RepoPath, RepoPathRef, RepoState,
 	},
 	sync::{BranchCompare, CommitId},
 	AsyncDiff, AsyncGitNotification, AsyncStatus, DiffParams,
@@ -67,6 +66,8 @@ pub struct Status {
 	index_wd: ChangesComponent,
 	diff: DiffComponent,
 	git_diff: AsyncDiff,
+	has_remotes: bool,
+	git_state: RepoState,
 	git_status_workdir: AsyncStatus,
 	git_status_stage: AsyncStatus,
 	git_branch_state: Option<BranchCompare>,
@@ -161,6 +162,8 @@ impl Status {
 		Self {
 			queue: queue.clone(),
 			visible: true,
+			has_remotes: false,
+			git_state: RepoState::Clean,
 			focus: Focus::WorkDir,
 			diff_target: DiffTarget::WorkingDir,
 			index_wd: ChangesComponent::new(
@@ -295,38 +298,34 @@ impl Status {
 		f: &mut tui::Frame<B>,
 		r: tui::layout::Rect,
 	) {
-		if let Ok(state) = sync::repo_state(&self.repo.borrow()) {
-			if state != RepoState::Clean {
-				let txt = Self::repo_state_text(
-					&self.repo.borrow(),
-					&state,
-				);
+		if self.git_state != RepoState::Clean {
+			let txt = Self::repo_state_text(
+				&self.repo.borrow(),
+				&self.git_state,
+			);
 
-				let w = Paragraph::new(txt)
-					.block(
-						Block::default()
-							.border_type(BorderType::Plain)
-							.borders(Borders::all())
-							.border_style(
-								Style::default().fg(Color::Yellow),
-							)
-							.title(format!("Pending {:?}", state)),
-					)
-					.style(Style::default().fg(Color::Red))
-					.alignment(Alignment::Left);
+			let w = Paragraph::new(txt)
+				.block(
+					Block::default()
+						.border_type(BorderType::Plain)
+						.borders(Borders::all())
+						.border_style(
+							Style::default().fg(Color::Yellow),
+						)
+						.title(format!(
+							"Pending {:?}",
+							self.git_state
+						)),
+				)
+				.style(Style::default().fg(Color::Red))
+				.alignment(Alignment::Left);
 
-				f.render_widget(w, r);
-			}
+			f.render_widget(w, r);
 		}
 	}
 
 	fn repo_state_unclean(&self) -> bool {
-		if let Ok(state) = sync::repo_state(&self.repo.borrow()) {
-			if state != RepoState::Clean {
-				return true;
-			}
-		}
-		false
+		self.git_state != RepoState::Clean
 	}
 
 	fn can_focus_diff(&self) -> bool {
@@ -409,6 +408,9 @@ impl Status {
 				config,
 			))?;
 
+			self.git_state = sync::repo_state(&self.repo.borrow())
+				.unwrap_or(RepoState::Clean);
+
 			self.branch_compare();
 		}
 
@@ -422,6 +424,13 @@ impl Status {
 			|| self.git_status_workdir.is_pending()
 	}
 
+	fn check_remotes(&mut self) {
+		self.has_remotes =
+			sync::get_branches_info(&self.repo.borrow(), false)
+				.map(|branches| !branches.is_empty())
+				.unwrap_or(false);
+	}
+
 	///
 	pub fn update_git(
 		&mut self,
@@ -449,6 +458,7 @@ impl Status {
 		self.index_wd.set_items(&workdir_status.items)?;
 
 		self.update_diff()?;
+		self.check_remotes();
 
 		if self.git_action_executed {
 			self.git_action_executed = false;
@@ -568,12 +578,6 @@ impl Status {
 		}
 	}
 
-	fn has_remotes(&self) -> bool {
-		get_branches_info(&self.repo.borrow(), false)
-			.map(|l| !l.is_empty())
-			.unwrap_or(false)
-	}
-
 	fn pull(&self) {
 		if let Some(branch) = self.git_branch_name.last() {
 			self.queue.push(InternalEvent::Pull(branch));
@@ -603,29 +607,23 @@ impl Status {
 		self.git_branch_state
 			.as_ref()
 			.map_or(true, |state| state.ahead > 0)
-			&& self.has_remotes()
+			&& self.has_remotes
 	}
 
-	fn can_pull(&self) -> bool {
-		self.has_remotes() && self.git_branch_state.is_some()
+	const fn can_pull(&self) -> bool {
+		self.has_remotes && self.git_branch_state.is_some()
 	}
 
 	fn can_abort_merge(&self) -> bool {
-		sync::repo_state(&self.repo.borrow())
-			.unwrap_or(RepoState::Clean)
-			== RepoState::Merge
+		self.git_state == RepoState::Merge
 	}
 
 	fn pending_rebase(&self) -> bool {
-		sync::repo_state(&self.repo.borrow())
-			.unwrap_or(RepoState::Clean)
-			== RepoState::Rebase
+		self.git_state == RepoState::Rebase
 	}
 
 	fn pending_revert(&self) -> bool {
-		sync::repo_state(&self.repo.borrow())
-			.unwrap_or(RepoState::Clean)
-			== RepoState::Revert
+		self.git_state == RepoState::Revert
 	}
 
 	pub fn revert_pending_state(&self) {
@@ -937,6 +935,7 @@ impl Component for Status {
 
 	fn show(&mut self) -> Result<()> {
 		self.visible = true;
+		self.check_remotes();
 		self.update()?;
 
 		Ok(())
